import numpy as np
from geographiclib.constants import Constants
from geographiclib.geodesic import Geodesic

################################
# new version of the offset    #
# using TOPAZ grid coordinates #
################################

# compute the new position
def offset(x, y, u, v, time):
  # x, y: initial position
  # u: zonal component of the velocity
  # v: meridional component of the velocity
  # time: needed to compute the distance
  x1 = x + ((u/1000)*time)
  y1 = y + ((v/1000)*time) 
  return x1, y1

#############################
# new version of the offset #
#############################

# 1) compute azimuth and distance from u and v
def convertUVtoSpeedAndDirection(u,v,time):
  # u: zonal component of the velocity
  # v: meridional component of the velocity
  # time: needed to compute the distance
  azimuth = np.arctan2(-1.0*v, -1.0*u) * 360/2/np.pi
  distance = np.sqrt((u*time)**2 + (v*time)**2)
  if azimuth<0:
    azimuth = azimuth + 360
  azimuth = 360 - azimuth + 0
  if azimuth>360:
    azimuth = azimuth - 360
  return azimuth, distance

# 2) compute the new position
def offset2(lon1, lat1, azimuth, distance):
  # lon1, lat1: initial position
  # azimuth, distance computed from convertUVtoSpeedAndDirection(u,v,time)
  d = Geodesic.WGS84.Direct(lat1, lon1, azimuth, distance)
  return np.asarray([d['lon2']]), np.asarray([d['lat2']])

#############################
# old version of the offset #
#############################
def offset3(u,v,lon,lat,time):
  # u: zonal component of the velocity
  # v: meridional component of the velocity
  # lon, lat: initial position
  # time: needed to compute the distance

  R=6378.137 # Reference radius of earth [km]
  # compute the azimuth theta (in radians, clockwise from north);
  theta=np.arctan2(v,u)
  # conversion from cckwise to cwise 
  if theta>0:
    theta=(2.*np.pi)-theta
  if theta<0:
    theta=-1.*theta
  # conversion from East reference to North reference
  if theta<(3.*np.pi)/2:
    theta=theta+(np.pi/2)
  if theta>=(3.*np.pi)/2:
    theta=theta-(3.*np.pi)/2
  # distance in km (velocity*time)
  d=np.sqrt((u*time/1000)**2 + (v*time/1000)**2)
  # degrees to radian
  lat=lat*(np.pi/180)
  lon=(lon+180)*(np.pi/180)
  # d/R is the angular distance (in radians), where d is the distance travelled and R is the earthradius
  lat2 = np.arcsin(np.sin(lat)*np.cos(d/R)+np.cos(lat)*np.sin(d/R)*np.cos(theta))
  lon2 = lon + np.arctan2(np.sin(theta)*np.sin(d/R)*np.cos(lat),np.cos(d/R)-np.sin(lat)*np.sin(lat2))
  # back to degrees
  lat=lat*(180/np.pi);
  lon=lon*(180/np.pi);
  lat2=lat2*(180/np.pi);
  lon2=lon2*(180/np.pi);

  offlon=lon2-lon
  offlat=lat2-lat
  
  return offlon,offlat

