import matplotlib
matplotlib.use('Agg')
from mpl_toolkits.basemap import Basemap
import pyproj as pyproj
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
from mpl_toolkits.axes_grid1.inset_locator import mark_inset
import numpy as np
import numpy.ma as ma

def plot_track(BuoyID,date,file_out,lons,lats,new_lon_M1,new_lat_M1,new_lon_M2,new_lat_M2,true_lon,true_lat):
  fig=plt.figure()
  ax = fig.add_subplot(111)
  lon_0 = lons.mean()
  lat_0 = lats.mean()
  minlon=np.nanmin([new_lon_M1,new_lon_M2,true_lon])
  minlat=np.nanmin([new_lat_M1,new_lat_M2,true_lat])
  maxlon=np.nanmax([new_lon_M1,new_lon_M2,true_lon])
  maxlat=np.nanmax([new_lat_M1,new_lat_M2,true_lat])
  minlon=minlon-(maxlon-minlon)/3
  maxlon=maxlon+(maxlon-minlon)/3
  minlat=minlat-(maxlat-minlat)/3
  maxlat=maxlat+(maxlat-minlat)/3
  #minlon=minlon-1
  #maxlon=maxlon+1
  #minlat=minlat-1
  #maxlat=maxlat+1
  map = Basemap(projection='npstere',boundinglat=60,lat_0=lat_0,lon_0=lon_0,resolution='l',llcrnrlon=minlon,llcrnrlat=minlat,urcrnrlon=maxlon, urcrnrlat=maxlat)
  map.drawcoastlines(linewidth=0.25)
  map.drawcountries(linewidth=0.25)
  map.drawmeridians(np.arange(0,360,30),linewidth=0.25,labels=[False,True,True,False])
  map.drawparallels(np.arange(75,90,5),linewidth=0.25,labels=[True,False,False,True])
  plt.title('Trajectories of Buoy ' + BuoyID + ""+ " starting from " + date, y=1.08)
  x1, x2, y1, y2 = minlon,maxlon,minlat,maxlat
  x1,y1=map(x1,y1)
  x2,y2=map(x2,y2)
  if np.abs(x1)<90:
    loc_=1 ; loc1_=3; loc2_=4  # zoom upper-right 
  if np.abs(x1)>=90:
    loc_=4 ; loc1_=1 ; loc2_=2 # zoom bottom-right 
  h1, =ax.plot([1,1],'r-',linewidth=2,label='\"Push-the-Stone\"')
  h2, =ax.plot([1,1],'b-',linewidth=2,label='Runge-Kutta 2')
  h3, =ax.plot([1,1],'g-',linewidth=2,label='Observations')
  plt.legend(loc='upper right',ncol=1, mode="expand", borderaxespad=0.,frameon=True)
  h1.set_visible(False)
  h2.set_visible(False)
  h3.set_visible(False)
  axins = inset_axes(ax, width=1.5,height=1.5, loc=loc_)
  print x1, x2, y1, y2
  axins.axis([x1, x2, y1, y2])
  new_lon_M1,new_lat_M1=map(new_lon_M1,new_lat_M1)
  new_lon_M2,new_lat_M2=map(new_lon_M2,new_lat_M2)
  true_lon,true_lat=map(true_lon,true_lat)
  true_lon_m=ma.masked_where(np.isnan(true_lon),true_lon) 
  true_lat_m=ma.masked_where(np.isnan(true_lat),true_lat) 
  plt.plot(new_lon_M1,new_lat_M1,'r-',linewidth=0.5,label='\"Push-the-Stone\"')
  plt.plot(new_lon_M2,new_lat_M2,'b-',linewidth=0.5,label='Runge-Kutta 2')
  plt.plot(true_lat_m,true_lat_m,'g-',linewidth=0.5,label='Observations')
  plt.plot(new_lon_M1,new_lat_M1,'ro',markersize=3)
  plt.plot(new_lon_M2,new_lat_M2,'bo',markersize=3)
  plt.plot(true_lon_m,true_lat_m,'go',markersize=3)
  plt.plot(new_lon_M1[0], new_lat_M1[0],'H',color="black",markersize=3)
  plt.xticks(visible=False)
  plt.yticks(visible=False)
  axins.xaxis.set_ticks_position('none')
  axins.yaxis.set_ticks_position('none')
  mark_inset(ax, axins, loc1=loc1_, loc2=loc2_, fc="none", color="red",linestyle="dashed",linewidth=0.5,ec="0.5")
  plt.savefig(file_out, dpi=300)
  plt.close()
