import numpy as np
import numpy.ma as ma
import scipy as sp
from bilin_interp import *
from scipy.interpolate import RectBivariateSpline
from offset import *
import imp
import offset ; imp.reload(offset) ; from offset import *
from proj_stereo import *

def rk2(u,v,x,y,x_,y_,delt):

  # u is the velocity u-component
  # v is the velocity v-component 
  # x and y are the initial coordinates of the buoy 
  # x_ and y_ are the coordinates of the (regular) TOPAZ grid 
  # delt is the integration step

  regx=np.sort(np.unique(x_)) 
  regy=np.sort(np.unique(y_))
  stepX=np.unique(np.diff(regx))[0]
  stepY=np.unique(np.diff(regy))[0]
  minX=np.min(regx)
  minY=np.min(regy)
  nX=len(regx)
  nY=len(regy)

  # Cycle time steps of this day 
  for istep2 in range(1,int(1/(delt/86400.))+1):
    lon, lat= polar_stereographic(x,y,'xy2ll')
    xpred=x # position initiale en x 
    ypred=y # position initiale en y
    # Runge-Kutta time step
    for irk in range(1,3):  
      if (np.isnan(x)==False):
        # bilinear coefficients for t=1,2
        f2d = RectBivariateSpline(regy,regx,u[0,:,:], kx=1,ky=1)
        urk1 = f2d.ev(ypred,xpred)
        f2d = RectBivariateSpline(regy,regx,u[1,:,:], kx=1,ky=1)
        urk2 = f2d.ev(ypred,xpred)
        f2d = RectBivariateSpline(regy,regx,v[0,:,:], kx=1,ky=1)
        vrk1 = f2d.ev(ypred,xpred)
        f2d = RectBivariateSpline(regy,regx,v[1,:,:], kx=1,ky=1)
        vrk2 = f2d.ev(ypred,xpred)

      else:
        urk1=0. ; vrk1=0. ; urk2=0. ; vrk2=0.
      
      # linear interpolation in time
      if (irk==1):
        wt=((istep2-1)*delt)/86400.
      else:
        wt=((istep2-1)*delt)/86400.+0.125 #??
      urk=(1.-wt)*urk1 + wt*urk2
      vrk=(1.-wt)*vrk1 + wt*vrk2
      # set value for rkdt 
      if (irk==1):
        rkdt=1/2
      else:
        rkdt=1

      # Estimate position 
      if (irk==1): # irk==1 : middle point
        xpred, ypred = offset(xpred,ypred,urk,vrk,delt/2)
      else:        # irk==2 : final position
        x, y = offset(x,y,urk,vrk,delt)
    # fin loop irk
  # fin loop istep2

  return x,y
